"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../constants");
const outputConstants_1 = require("./outputConstants");
const pkgsDiff_1 = __importStar(require("./pkgsDiff"));
const path = require("path");
const chalk = require("chalk");
const most = require("most");
const R = require("ramda");
const semver = require("semver");
exports.default = (log$, opts) => {
    const pkgsDiff$ = pkgsDiff_1.default(log$, { prefix: opts.cwd });
    const summaryLog$ = log$.summary
        .take(1);
    return most.combine((pkgsDiff) => {
        var _a;
        let msg = '';
        for (const depType of ['prod', 'optional', 'peer', 'dev', 'nodeModulesOnly']) {
            const diffs = R.values(pkgsDiff[depType]);
            if (diffs.length) {
                msg += constants_1.EOL;
                if ((_a = opts.pnpmConfig) === null || _a === void 0 ? void 0 : _a.global) {
                    msg += chalk.cyanBright(`${opts.cwd}:`);
                }
                else {
                    msg += chalk.cyanBright(`${pkgsDiff_1.propertyByDependencyType[depType]}:`);
                }
                msg += constants_1.EOL;
                msg += printDiffs(diffs, { prefix: opts.cwd });
                msg += constants_1.EOL;
            }
        }
        return { msg };
    }, pkgsDiff$, summaryLog$)
        .take(1)
        .map(most.of);
};
function printDiffs(pkgsDiff, opts) {
    // Sorts by alphabet then by removed/added
    // + ava 0.10.0
    // - chalk 1.0.0
    // + chalk 2.0.0
    pkgsDiff.sort((a, b) => (a.name.localeCompare(b.name) * 10 + (Number(!b.added) - Number(!a.added))));
    const msg = pkgsDiff.map((pkg) => {
        let result = pkg.added
            ? outputConstants_1.ADDED_CHAR
            : outputConstants_1.REMOVED_CHAR;
        if (!pkg.realName || pkg.name === pkg.realName) {
            result += ` ${pkg.name}`;
        }
        else {
            result += ` ${pkg.name} <- ${pkg.realName}`;
        }
        if (pkg.version) {
            result += ` ${chalk.grey(pkg.version)}`;
            if (pkg.latest && semver.lt(pkg.version, pkg.latest)) {
                result += ` ${chalk.grey(`(${pkg.latest} is available)`)}`;
            }
        }
        if (pkg.deprecated) {
            result += ` ${chalk.red('deprecated')}`;
        }
        if (pkg.from) {
            result += ` ${chalk.grey(`<- ${pkg.from && path.relative(opts.prefix, pkg.from) || '???'}`)}`;
        }
        return result;
    }).join(constants_1.EOL);
    return msg;
}
//# sourceMappingURL=reportSummary.js.map